﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;
using System.Collections.Generic;

namespace MvcEFCodeFirst.Data
{
  [AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
  public class AdvancedEmailValidationAttribute : ValidationAttribute, IClientValidatable
  {
    public AdvancedEmailValidationAttribute()
      : this(0)
    {
    }

    public AdvancedEmailValidationAttribute(int maximumLength)
    {
      MaximumLength = maximumLength == 0 ?
        50 : maximumLength;
      ErrorMessage = string.Format("Email cannot contain first or last name and cannot be longer than {0} characters.", MaximumLength);
    }
    public int MaximumLength { get; private set; }

    protected override ValidationResult IsValid(object value, ValidationContext validationContext)
    {
      var objectToValidate = validationContext.ObjectInstance as IPersonWithEmail;
      ValidationResult returnValue = ValidationResult.Success;

      if (objectToValidate != null)
      {
        if (value != null)
        {
          if (
            value.ToString().ToUpper().Contains(objectToValidate.LastName.ToUpper()) ||
            value.ToString().ToUpper().Contains(objectToValidate.LastName.ToUpper()) || 
            value.ToString().Length > MaximumLength)
          {
            returnValue = new ValidationResult(ErrorMessage);
          }
        }
      }
      else
      {
        returnValue = new ValidationResult("You must implement IPersonWithEmail on your object to use this rule");
      }
      return returnValue;
    }

    public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
    {
      var rule = new ModelClientValidationRule
      {
        ErrorMessage = ErrorMessage,
        ValidationType = "advancedemail"
      };
      rule.ValidationParameters.Add("maxlength", MaximumLength);
      rule.ValidationParameters.Add("firstname", "FirstName");
      rule.ValidationParameters.Add("lastname", "LastName");
      yield return rule;

    }
  }
}