﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;
using MvcEFCodeFirst.Resources;
using System.Web.Mvc;

namespace MvcEFCodeFirst.Data
{
  public class Attendee : IPersonWithEmail
  {
    [Key]
    public int AttendeeID { get; set; }

    [StringLength(50,
      ErrorMessageResourceName = "FirstName50Chars",
      ErrorMessageResourceType = typeof(Resource))]
    [Required(
      ErrorMessageResourceName = "FirstNameRequired",
      ErrorMessageResourceType = typeof(Resource))]
    [Display(Name = "FirstName", ResourceType = typeof(Resource))]
    public string FirstName { get; set; }

    [StringLength(50,
      ErrorMessageResourceName = "LastName50Chars",
      ErrorMessageResourceType = typeof(Resource))]
    [Required(
      ErrorMessageResourceName = "LastNameRequired",
      ErrorMessageResourceType = typeof(Resource))]
    [Display(Name = "LastName", ResourceType = typeof(Resource))]
    public string LastName { get; set; }

    [StringLength(250)]
    [Required(
     ErrorMessageResourceName = "EmailRequired",
     ErrorMessageResourceType = typeof(Resource))]
    [RegularExpression(
      @"^[A-Za-z0-9](([_\.\-]?[a-zA-Z0-9]+)*)@([A-Za-z0-9]+)(([\.\-]?[a-zA-Z0-9]+)*)\.([A-Za-z]{2,})$",
      ErrorMessageResourceName = "EmaiInvalidFormat",
      ErrorMessageResourceType = typeof(Resource))]
    [Display(Name = "Email", ResourceType = typeof(Resource))]
    [Remote(
      "DuplicateEmail",
      "Attendee",
      AdditionalFields = "AttendeeID",
      ErrorMessageResourceName = "DuplicateEmail",
      ErrorMessageResourceType = typeof(Resource))]
    [AdvancedEmailValidation()]
    public string Email { get; set; }

    [StringLength(int.MaxValue)]
    [Display(Name = "Notes", ResourceType = typeof(Resource))]
    public string Notes { get; set; }

    public virtual ICollection<Session> Sessions { get; set; }
  }
}
