﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcEFCodeFirst.Data;
using MvcEFCodeFirst.DataAccess;
using System.ComponentModel;

namespace MvcEFCodeFirst.Controllers
{
  public class AttendeeController : AsyncController
  {
    private CodeStockRepository db = new CodeStockRepository();

    //
    // GET: /Attendee/

    public void IndexAsync()
    {
      AsyncManager.OutstandingOperations.Increment();
      BackgroundWorker worker = new BackgroundWorker();
      worker.DoWork += (o, e) =>
      {
        AsyncManager.Parameters["attendees"] = db.Select<Attendee>().OrderBy(_ => _.LastName).ThenBy(_ => _.FirstName).ToList();
      };
      worker.RunWorkerCompleted += (o, e) => { AsyncManager.OutstandingOperations.Decrement(); };
      worker.RunWorkerAsync();

    }


    public ViewResult IndexCompleted(IEnumerable<Attendee> attendees)
    {
      return View(attendees);
    }

    //
    // GET: /Attendee/Details/5

    public ViewResult Details(int id)
    {
      Attendee attendee = db.Find<Attendee>(id);
      return View(attendee);
    }

    public JsonResult DuplicateEmail(string email, int? attendeeID)
    {
      int id = attendeeID.HasValue ? attendeeID.Value : 0;
      return Json(!db.Select<Attendee>()
        .Where(one => one.Email == email && one.AttendeeID != id).Any(), 
        JsonRequestBehavior.AllowGet);
    }

    //
    // GET: /Attendee/Create

    public ActionResult Create()
    {
      ViewBag.Title = "Create";
      ViewBag.ButtonText = "Create";
      return View("CreateEdit", new Attendee());
    }

    //
    // POST: /Attendee/Create

    [HttpPost]
    public ActionResult Create(Attendee attendee)
    {
      if (ModelState.IsValid)
      {
        db.Insert(attendee);
        return RedirectToAction("Index");
      }

      return View(attendee);
    }

    [HttpPost]
    public ActionResult CreateUpdateAjax(Attendee attendee)
    {
      if (ModelState.IsValid)
      {
        if (attendee.AttendeeID == 0)
        {
          db.Insert(attendee);
        }
        else
        {
          db.SetModified(attendee);
          db.Save();
        }
      }
      return PartialView("AttendeePartial", attendee);
    }

    //
    // GET: /Attendee/Edit/5

    public ActionResult Edit(int id)
    {
      ViewBag.Title = "Edit";
      ViewBag.ButtonText = "Save";
      Attendee attendee = db.Find<Attendee>(id);
      return View("CreateEdit", attendee);
    }

    //
    // POST: /Attendee/Edit/5

    [HttpPost]
    public ActionResult Edit(Attendee attendee)
    {
      if (ModelState.IsValid)
      {
        db.SetModified(attendee);
        db.Save();
      }
      return PartialView("AttendeePartial", attendee);
    }

    //
    // GET: /Attendee/Delete/5

    public ActionResult Delete(int id)
    {
      Attendee attendee = db.Find<Attendee>(id);
      return View(attendee);
    }

    //
    // POST: /Attendee/Delete/5

    [HttpPost, ActionName("Delete")]
    public ActionResult DeleteConfirmed(int id)
    {
      Attendee attendee = db.Find<Attendee>(id);
      db.Delete(attendee);
      return RedirectToAction("Index");
    }

    protected override void Dispose(bool disposing)
    {
      db.Dispose();
      base.Dispose(disposing);
    }
  }
}