﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcEFCodeFirst.Data;
using MvcEFCodeFirst.DataAccess;
using MvcEFCodeFirst.Models;
using MvcSampleApp.Helpers;
using MvcEFCodeFirst.Resources;

namespace MvcEFCodeFirst.Controllers
{
  public class SessionController : Controller
  {
    private CodeStockContext db = new CodeStockContext();

    //
    // GET: /Session/

    public ViewResult Index()
    {
      return View(db.Sessions.ToList());
    }

    //
    // GET: /Session/Details/5

    public ViewResult Details(int id)
    {
      Session session = db.Sessions.Find(id);
      return View(session);
    }

    //
    // GET: /Session/Create

    public ActionResult Create()
    {
      var view = View(new Session());
      view.ViewBag.Action = Resource.Create;
      return view;
    }

    //
    // POST: /Session/Create

    [HttpPost]
    public ActionResult Create(Session session)
    {
      if (ModelState.IsValid)
      {
        db.Sessions.Add(session);
        db.SaveChanges();
        return RedirectToAction("Index");
      }
      else
      {
        if (ModelState["SessionDate"].Errors.Count == 1)
        {
          string date = ModelState["SessionDate"].Value.AttemptedValue;
          DateTime test;
          if (!DateTime.TryParse(date, out test))
          {
            ModelState["SessionDate"].Errors.Clear();
            ModelState["SessionDate"].Errors.Add(new ModelError(Resource.DateInvalidInvalidFormat));
          }
        }
      }
      var view = View(session);
      view.ViewBag.Action = Resource.Create;
      return view;
    }

    //
    // GET: /Session/Edit/5

    public ActionResult Edit(int id)
    {
      Session session = db.Sessions.Find(id);
      var view = View(session);
      view.ViewBag.Action = Resource.Save;
      return view;
    }

    //
    // POST: /Session/Edit/5

    [HttpPost]
    public ActionResult Edit(Session session)
    {
      if (ModelState.IsValid)
      {
        db.Entry(session).State = EntityState.Modified;
        db.SaveChanges();
        return RedirectToAction("Index");
      }
      return View(session);
    }

    //
    // GET: /Session/Delete/5

    public ActionResult Delete(int id)
    {
      Session session = db.Sessions.Find(id);
      var view = View(session);
      view.ViewBag.Action = Resource.Delete;
      return view;
    }

    //
    // POST: /Session/Delete/5

    [HttpPost, ActionName("Delete")]
    public ActionResult DeleteConfirmed(int id)
    {
      Session session = db.Sessions.Find(id);
      db.Sessions.Remove(session);
      db.SaveChanges();
      return RedirectToAction("Index");
    }

    protected override void Dispose(bool disposing)
    {
      db.Dispose();
      base.Dispose(disposing);
    }

    /// <summary>
    /// Get Json data for the grid
    /// </summary>
    /// <param name="sidx">Column to sort on.  Do not change parameter name</param>
    /// <param name="sord">Sort direction.  Do not change parameter name</param>
    /// <param name="page">Current page.  Do not change parameter name</param>
    /// <param name="rows">Number of rows to get.  Do not change parameter name</param>
    /// <returns></returns>
    public ActionResult SessionsGridData(string sidx, string sord, int? page, int? rows)
    {

      int pageSize = rows ?? 2;
      int totalRecords = db.Sessions.Count();
      int totalPages = (int)Math.Ceiling((float)totalRecords / (float)pageSize);
      int currentPage = page ?? 1;
      string sortByColumnName = sidx ?? "SessionDate";
      string sortDirection = sord ?? "desc";

      var sessions = (from one in db.Sessions
                      select one);
      if (sortByColumnName == "SessionDate")
      {
        if (sortDirection.Equals("desc"))
          sessions = sessions.OrderByDescending(blog => blog.SessionDate);
        else
          sessions = sessions.OrderBy(blog => blog.SessionDate);
      }
      else
      {
        if ("desc".Equals(sortDirection))
          sessions = sessions.OrderByDescending(session => session.Title);
        else
          sessions = sessions.OrderBy(session => session.Title);
      }

      var data = sessions.Skip((currentPage - 1) * pageSize).Take(pageSize).AsNoTracking().ToArray();

      var jsonData = new jqGridJson()
      {
        total = totalPages,
        page = currentPage,
        records = totalRecords,
        rows = (
          from session in data
          select new jqGridRowJson
          {
            id = session.SessionID.ToString(),
            cell = new string[] { 
              "<a href='" + string.Format("./Session/Details/{0}", session.SessionID.ToString()) + "'>" + Resource.Details + "</a>",
              "<a href='" + string.Format("./Session/Edit/{0}", session.SessionID.ToString()) + "'>" + Resource.Edit + "</a>",
              "<a href='" + string.Format("./Session/Delete/{0}", session.SessionID.ToString()) + "'>" + Resource.Delete +"</a>",
              session.Title, 
              session.SessionDate.Value.ToShortDateString() }
          }).ToArray()
      };

      return Json(jsonData, JsonRequestBehavior.AllowGet);
    }
  }
}